from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
import json


class Api(object):

    def __init__(self, options):
        pass

    def start(self):
        pass

    def halt(self):
        pass


class HttpRequestHandler(BaseHTTPRequestHandler):

    def do_OPTIONS(self):
        self.send_response(204)

        self.send_header('Access-Control-Allow-Origin', '*')
        self.send_header('Access-Control-Allow-Headers', 'Content-Type')
        self.end_headers()

    def do_GET(self):
        import traceback

        if not hasattr(self, "method"):
            self.method = "GET"

        self.send_response(200)

        path_parts = filter(len, self.path.split("/"))

        try:
            response = self.get_response(path_parts)
        except Exception as ex:
            print(traceback.format_exc())

            response = {
                "error": str(ex),
            }

        json_response = json.dumps(response)

        self.send_header('Content-Type', 'application/json')
        self.send_header('Content-Length', len(json_response))
        self.send_header('Access-Control-Allow-Origin', '*')
        self.end_headers()

        self.wfile.write(json_response)
        self.wfile.close()

    def do_POST(self):
        self.method = "POST"

        return self.do_GET()

    def get_response(self, path):
        if not path or path[0] != "api":
            return {}

        if len(path) == 1:
            return self.handle_api()

        if len(path) == 2 and path[1] == "robots":
            return self.handle_robots()

        if len(path) == 3 and path[1] == "robots":
            return self.handle_robot(path[2])

        if len(path) == 4 and path[1] == "robots":
            if path[3] == "devices":
                return self.handle_robot_devices(path[2])

        if len(path) == 5 and path[1] == "robots":
            if path[3] == "devices":
                return self.handle_robot_device(path[2], path[4])

        if len(path) == 6 and path[1] == "robots":
            if path[3] == "devices":
                if path[5] == "commands":
                    return self.handle_robot_device_commands(path[2], path[4])

                if path[5] == "events":
                    return self.handle_robot_device_events(path[2], path[4])

        if len(path) == 7 and path[1] == "robots":
            if path[3] == "devices":
                if path[5] == "commands":
                    return self.handle_robot_device_command(
                        path[2], path[4], path[6]
                    )

        return {}

    def handle_api(self):
        response = {
            "commands": [],
            "events": [],
        }

        response.update(self.handle_robots())

        return {
            "MCP": response,
        }

    def handle_robots(self):
        from zorg import main

        robots = main.robots

        response = {
            "robots": [],
        }

        for robot_name, robot in robots.items():
            serialized = robot.serialize()

            response["robots"].append(serialized)

        return response

    def handle_robot(self, name):
        from zorg import main

        robots = main.robots
        robot = robots[name]

        response = {
            "robot": robot.serialize(),
        }

        return response

    def handle_robot_devices(self, name):
        from zorg import main

        robots = main.robots
        robot = robots[name]

        response = {
            "devices": robot.serialize_devices(),
        }

        return response

    def handle_robot_device(self, robot_name, device_name):
        from zorg import main

        robots = main.robots
        robot = robots[robot_name]

        device = getattr(robot.helper, device_name)

        return {
            "device": device.serialize()
        }

    def handle_robot_device_commands(self, robot_name, device_name):
        from zorg import main

        robots = main.robots
        robot = robots[robot_name]

        device = getattr(robot.helper, device_name)

        return {
            "commands": device.commands,
        }

    def handle_robot_device_events(self, robot_name, device_name):
        from zorg import main

        robots = main.robots
        robot = robots[robot_name]

        device = getattr(robot.helper, device_name)

        return {
            "events": device.events,
        }

    def handle_robot_device_command(
            self, robot_name, device_name, command_name):
        from zorg import main

        robots = main.robots
        robot = robots[robot_name]

        device = getattr(robot.helper, device_name)

        command = getattr(device, command_name)

        if self.method == "GET":
            request_body = ""
        else:
            request_body = self.rfile.read(
                int(self.headers.getheader('content-length'))
            )

        if request_body:
            args = json.loads(request_body)
        else:
            args = {}

        result = command(**args)

        return {
            "result": result,
        }


class Http(Api):

    def start(self):
        server = HTTPServer(('0.0.0.0', 8000), HttpRequestHandler)
        server.serve_forever()



import importlib


def import_class(module_path):
    module_parts = module_path.split(".")
    module_path = ".".join(module_parts[:-1])
    module = importlib.import_module(module_path)

    return getattr(module, module_parts[-1])



from multiprocessing import Process
from .connection import Connection
from .device import Device


class Helper(object):

    def __init__(self, devices, connections):
        self.devices = devices
        self.connections = connections

        self._devices = {}
        self._connections = {}

    def __getattr__(self, name):
        if name in self._devices:
            return self._devices[name]

        self._devices[name] = self.initialize_device(name)

        return self._devices[name]

    def initialize_device(self, name):
        device_config = self.devices[name]

        connection_name = device_config["connection"]

        if connection_name in self._connections:
            connection = self._connections[connection_name]
        else:
            connection = self.initialize_connection(connection_name)
            self._connections[connection_name] = connection

        device = Device(device_config, connection)

        return device

    def initialize_connection(self, name):
        connection_config = self.connections[name]

        connection = Connection(connection_config)

        return connection


class Robot(object):

    def __init__(self, options):
        self.name = options.get("name", "")
        self.connections = options.get("connections", {})
        self.adaptors = options.get("adaptors", {})
        self.devices = options.get("devices", {})

        self.drivers = {}

        self.commands = []
        self.events = []

        self.running = False

        self.work = options["work"]

        self.helper = Helper(self.devices, self.connections)

    def start(self):
        process = Process(target=self.work, args=(self.helper, ))

        process.start()
        return process

    def halt(self):
        pass

    def serialize(self):
        return {
            "name": self.name,
            "commands": self.commands,
            "events": self.events,
            "connections": self.serialize_connections(),
            "devices": self.serialize_devices(),
        }

    def serialize_connections(self):
        connections = []

        for name, config in self.connections.items():
            config["name"] = name
            connection = Connection(config)

            connections.append(connection.serialize())

        return connections

    def serialize_devices(self):
        devices = []

        for name, config in self.devices.items():
            config["name"] = name
            device = Device(config, None)

            devices.append(device.serialize())

        return devices



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='zorg',
    version='0.0.4',
    url='https://github.com/zorg/zorg',
    description='Python framework for robotics and physical computing.',
    long_description=read('README.rst'),
    author='Zorg Group',
    maintainer_email='gunthercx@gmail.com',
    packages=find_packages(),
    package_dir={'zorg': 'zorg'},
    include_package_data=True,
    license='MIT',
    zip_safe=True,
    platforms=['any'],
    keywords=['zorg', 'robotics'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=[]
)
